@@ -10,6 +10,7 @@ import v20
 from autotrader.brokers.oanda import utils
 import datetime
 import pandas as pd
+import numpy as np
 
 
 class Oanda():
@@ -54,7 +55,40 @@ class Oanda():
         response = self.api.order.list_pending(accountID = self.ACCOUNT_ID, 
                                           instrument=instrument)
         
-        return response
+        oanda_pending_orders = response.body['orders']
+        pending_orders = {}
+        
+        for order in oanda_pending_orders:
+            if order.type != 'TAKE_PROFIT' and order.type != 'STOP_LOSS':
+                new_order = {}
+                new_order['order_ID']           = order.id
+                new_order['order_type']         = order.type
+                new_order['order_stop_price']   = order.price
+                new_order['order_limit_price']  = order.price
+                new_order['direction']          = np.sign(order.units)
+                new_order['order_time']         = order.createTime
+                new_order['strategy']           = None
+                new_order['instrument']         = order.instrument
+                new_order['size']               = order.units
+                new_order['order_price']        = order.price
+                new_order['granularity']        = None
+                new_order['take_profit']        = order.takeProfitOnFill.price
+                new_order['take_distance']      = None
+                new_order['stop_type']          = None
+                new_order['stop_distance']      = None
+                new_order['stop_loss']          = None
+                new_order['related_orders']     = None
+                
+                if instrument is not None and order.instrument == instrument:
+                    pending_orders[order.id] = new_order
+                elif instrument is None:
+                    pending_orders[order.id] = new_order
+            
+        return pending_orders
+    
+    def cancel_pending_order(self, order_id):
+        ''' Cancels pending order by ID. '''
+        self.api.order.cancel(accountID = self.ACCOUNT_ID, orderSpecifier=str(order_id))
     
     def get_open_positions(self, pair = None):
         ''' Gets the current positions open on the account. '''
@@ -188,29 +222,32 @@ class Oanda():
         return response
     
     
-    def close_position(self, pair):
+    def close_position(self, instrument, long_units=None, short_units=None,
+                       **dummy_inputs):
         ''' Closes all open positions on an instrument '''
         # Check if the position is long or short
-        position    = self.get_positions(pair)['position']
-        long_units  = position.long.units
-        short_units = position.short.units
+        position    = self.get_positions(instrument)['position']
+        if long_units is None:
+            long_units  = position.long.units
+        if short_units is None:
+            short_units = position.short.units
         
         if long_units > 0:
             response = self.api.position.close(accountID=self.ACCOUNT_ID, 
-                                               instrument=pair, 
+                                               instrument=instrument, 
                                                longUnits="ALL")
             # # Check response
             # output = self.check_response(response)
         
         elif short_units > 0: 
             response = self.api.position.close(accountID=self.ACCOUNT_ID, 
-                                               instrument=pair,
+                                               instrument=instrument,
                                                shortUnits="ALL")
             # # Check response
             # output = self.check_response(response)
         
         else:
-            print("There is no current position with {} to close.".format(pair))
+            print("There is no current position with {} to close.".format(instrument))
             response = None
         
         return response
@@ -406,4 +443,9 @@ class Oanda():
         if reduced_granularity[1:] == '0':
             reduced_granularity = reduced_granularity[0] + '1'
         
-        return reduced_granularity
\ No newline at end of file
+        return reduced_granularity
+    
+    def get_NAV(self):
+        ''' Returns Net Asset Value of account. '''
+        response = self.api.account.get(accountID=self.ACCOUNT_ID)
+        return response.body["account"].NAV
