@@ -24,7 +24,7 @@ def send_order(order_details, order_response, mailing_list, host_email):
     
     pair            = order_details['instrument']
     size            = order_details['size']
-    price           = order_details['price']
+    price           = order_details['order_price']
     stop_loss       = order_details['stop_loss']
     take_profit     = order_details['take_profit']
     strategy_name   = order_details['strategy']
@@ -34,9 +34,17 @@ def send_order(order_details, order_response, mailing_list, host_email):
         val         = 2
     else:
         val         = 4
-    stop_pips       = round(abs(price - stop_loss)*10**val, 1)
-    take_pips       = round(abs(price - take_profit)*10**val, 1)
     
+    if stop_loss is not None:
+        stop_pips = round(abs(price - stop_loss)*10**val, 1)
+    else:
+        stop_pips = None
+    
+    if take_profit is not None:
+        take_pips = round(abs(price - take_profit)*10**val, 1)
+    else:
+        take_pips = None
+        
     # Round price levels
     stop_loss       = round(stop_loss, val+1)
     take_profit     = round(take_profit, val+1)
